package PureMessage::_sys;

use strict;
use XSLoader ();
use ActiveState::Bytes qw(bytes_parse);

use constant SOLARIS_8_CANT_DETERMINE => -2;

my $SHMMAX = 0;
my $SHMALL = 1;
my $SEMMNI = 2;
my $SEMMNS = 3;

my %SYSCTL = (
    $SHMMAX => {
        linux => 'shmmax',
        freebsd => 'shmmax',
        solaris => 'max-shm-memory',
    },
    $SHMALL => {
        linux => 'shmall',
        freebsd => 'shmall',
        solaris => undef,
    },
    $SEMMNI => {
        linux => 'semmni',
        freebsd => 'semmni',
        solaris => 'max-sem-ids',
    },
    $SEMMNS => {
        linux => 'semmns',
        freebsd => 'semmns',
        solaris => undef, 
    },
);

our $VERSION = '6.000';
XSLoader::load(__PACKAGE__, $VERSION);

# legacy
sub lic_kv {
    require PureMessage::Config;
    my $tmp = lic_file($PureMessage::Config::lic_file) || return;
    $tmp = {lic_features($tmp)}->{'PureMessage::Milter'} || return;
    return lic_feature_kv($tmp);
}

sub lic_desc {
    my %lic = PureMessage::_sys::lic_kv();
    my $lictext = "";
    if (%lic) {
	my @items;
	$lictext = "$lic{LicenseType} license";

	# Eight-nines is the special value for "infinity".  Older versions
	# of pmx treated non-existence or zero as a default value, so we
	# can't use that to mean infinity.
	my $n = $lic{NumberOfUsers};
	push @items, "$n users" if defined $n && $n > 0 && $n != 99_999_999;
	$n = $lic{NumberOfCPUs};
	push @items, "$n CPUs"  if defined $n && $n > 0 && $n != 99_999_999;
	$n = $lic{NumberOfHosts};
	push @items, "$n hosts" if defined $n && $n > 0 && $n != 99_999_999;

	$lictext .= " for " . join(", ", @items) if @items;

	my $exp = $lic{ExpirationDate};
	if ($exp && $exp ne "99-99-9999") {
	    $lictext .= " (expires $exp)" ;
	}
    }
    return $lictext;
}

sub get_sysctl {
    my $type = shift;
    my $value = undef;

    if ($^O eq 'linux') {
        $value = qx#/sbin/sysctl -n kernel.$SYSCTL{$type}{linux} 2>&-#;
    }
    elsif ($^O eq 'freebsd') {
        $value = qx#/sbin/sysctl -n kern.ipc.$SYSCTL{$type}{freebsd} 2>&-#;
    }
    elsif ($^O eq 'solaris' and $SYSCTL{$type}{solaris}) {
        my @result = qx#/usr/bin/prctl -n project.$SYSCTL{$type}{solaris} $$ 2>&-#;
        for my $line (@result) {
            if ($line =~ /privileged\s+(.*?)\s+/) {
                $value = bytes_parse($1);
            }
        }
    }
    chomp $value;
    return $value;
}

sub get_shmmax { 
    my $val = _get_shm_value($SHMMAX);

    if($val <= 0) {
        $val = get_sysctl($SHMMAX) || -1;
    }
    return $val;
}

sub get_shmall { 
    my $val = _get_shm_value($SHMALL);

    if($val <= 0) {
        $val = get_sysctl($SHMALL) || -1;
    }
    return $val;
}

sub get_semmni { 
    my $val = _get_shm_value($SEMMNI);

    if($val <= 0) {
        $val = get_sysctl($SEMMNI) || -1;
    }
    return $val;
}

sub get_semmns { 
    my $val = _get_shm_value($SEMMNS);

    if($val <= 0) {
        $val = get_sysctl($SEMMNS) || -1;
    }
    return $val;
}
    
sub _get_shm_value {
    my $kind = shift;

    my $shm_val = _get_shm_setting($kind);
    
    if ($shm_val == SOLARIS_8_CANT_DETERMINE) {
        # We were built on Solaris 8, so use commandline utils to get the value
        my @shm_map = ('SHMMAX', 'SHMALL', 'SEMMNI', 'SEMMNS');
        my $setting = $shm_map[$kind];

        # determine if we're on Solaris 10 SPARC
        my $uname_r = `uname -r`;  chomp $uname_r;
        my $uname_s = `uname -s`;  chomp $uname_s; $uname_s = lc($uname_s);

        # Solaris 10 SPARC
        if (($uname_s eq 'sunos') && ($uname_r eq '5.10')) {

            if ($setting eq 'SHMMAX' || $setting eq 'SEMMNI') {
                my $proj_key = '';
                if ($setting eq 'SHMMAX') { $proj_key = 'project.max-shm-memory'; }
                if ($setting eq 'SEMMNI') { $proj_key = 'project.max-sem-ids'; }
                my $cmd = "/usr/bin/prctl -P -n '$proj_key' $$";
                my $prctl_n = `$cmd`;  chomp $prctl_n;
                if ($prctl_n =~ /^$proj_key\sprivileged\s*(\d+).*/m) {
                    $shm_val = $1;
                }
            }

            if ($shm_val == SOLARIS_8_CANT_DETERMINE) {
                # If we didn't get the value from prctl, use the Solaris 10 default

                # XXX: Real defaults for SEMMNI and SEMMNS need to be found
                # my @shm_defaults = (1048576, 256, 10, 256);
                my @shm_defaults = (1048576, SOLARIS_8_CANT_DETERMINE, 512, 512);

                $shm_val = $shm_defaults[$kind];
            }

        # Solaris 8 & 9 SPARC
        } else {

            # There's no way of detecting SHMMAX on Solaris without being setuid
            # root, so use sysdef
            my $cmd = '/usr/sbin/sysdef';
            if (-e $cmd) {
                my $fh;
                open($fh, "$cmd|") or die "Could not open pipe to '$cmd': $!";
                while (my $line = <$fh>) {
                    if ($line =~ /^\s*(\d+).*$setting/) {
                        $shm_val = $1;
                        last;
                    }
                }
                close($fh) or die "Could not close pipe to '$cmd': $!";
            }

            if ($shm_val == SOLARIS_8_CANT_DETERMINE) {
                # If we didn't get the value from sysdef, use the Solaris 8 default
                # XXX: Real defaults for SEMMNI and SEMMNS need to be found
                # my @shm_defaults = (1048576, 256, 10, 256);
                my @shm_defaults = (1048576, SOLARIS_8_CANT_DETERMINE, 512, 512);

                $shm_val = $shm_defaults[$kind];
            }
        }
    }

    return $shm_val;
}

1;
